// test AF on the Ov5640 camera with Xiao ESP32S3 , BSP 2.0.14
// 
// hack and chop of original CameraWebServer demo.
// Open a new browser window with http://<your-esp-ip>/af.html 
// you get the button.
//
// Auto focus on web cam stream demo PJGlasso 5/13/25
//
#include "esp_camera.h"
#include <WiFi.h>
#include <Wire.h>
#include "ESP32_OV5640_AF.h"   

#define CAMERA_MODEL_XIAO_ESP32S3 // Has PSRAM
#include "camera_pins.h"
#include "autofocus_http.h"

// WiFi credentials
const char* ssid = "Seeeed";
const char* password = "Studio";

// Autofocus related
#define BOOT_BUTTON_PIN 0 
// BOOT button on XIAO ESP32S3


unsigned long last_af_time = 0;
const unsigned long af_debounce_time = 1000; // 1 second debounce
#define OV5640_ADDR 0x3C >> 1

void startCameraServer();
void setupLedFlash(int pin);
void trigger_ov5640_autofocus();
void i2cWriteReg(uint8_t addr, uint16_t reg, uint8_t data);
OV5640 ov5640;
void setup() {
  Serial.begin(9600);
  while(!Serial);
  Serial.setDebugOutput(true);
  Serial.println();

  pinMode(BOOT_BUTTON_PIN, INPUT_PULLUP);

  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  config.frame_size = FRAMESIZE_UXGA;
  config.pixel_format = PIXFORMAT_JPEG;
  config.grab_mode = CAMERA_GRAB_WHEN_EMPTY;
  config.fb_location = CAMERA_FB_IN_PSRAM;
  config.jpeg_quality = 12;
  config.fb_count = 1;

  if(config.pixel_format == PIXFORMAT_JPEG){
    if(psramFound()){
      config.jpeg_quality = 10;
      config.fb_count = 2;
      config.grab_mode = CAMERA_GRAB_LATEST;
    } else {
      config.frame_size = FRAMESIZE_SVGA;
      config.fb_location = CAMERA_FB_IN_DRAM;
    }
  } else {
    config.frame_size = FRAMESIZE_240X240;
#if CONFIG_IDF_TARGET_ESP32S3
    config.fb_count = 2;
#endif
  }

  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    Serial.printf("Camera init failed with error 0x%x", err);
    return;
  }
sensor_t *s = esp_camera_sensor_get();
if (s && s->id.PID == OV5640_PID) {
    ov5640.start(s);
    if (ov5640.focusInit() == 0) {
        Serial.println("OV5640 autofocus initialized.");
    } else {
        Serial.println("AF init failed.");
    }
}

  if(config.pixel_format == PIXFORMAT_JPEG){
    s->set_framesize(s, FRAMESIZE_QVGA);
  }

#if defined(LED_GPIO_NUM)
  setupLedFlash(LED_GPIO_NUM);
#endif

  WiFi.begin(ssid, password);
  WiFi.setSleep(false);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");

  startCameraServer();

  Serial.print("Camera Ready! Use 'http://");
  Serial.print(WiFi.localIP());
  Serial.println("' to connect");
}

void loop() {
  if (digitalRead(BOOT_BUTTON_PIN) == LOW) {
    unsigned long current_time = millis();
    if (current_time - last_af_time > af_debounce_time) {
      trigger_ov5640_autofocus();
      last_af_time = current_time;
    }
  }

  delay(100);
}

void trigger_ov5640_autofocus() {
  Serial.println("Triggering OV5640 autofocus manually...");
  i2cWriteReg(OV5640_ADDR, 0x3000, 0x20);
  delay(10);
  i2cWriteReg(OV5640_ADDR, 0x3000, 0x00);
  delay(10);
  i2cWriteReg(OV5640_ADDR, 0x3022, 0x08);
  delay(500);
}

void i2cWriteReg(uint8_t addr, uint16_t reg, uint8_t data) {
  Wire.beginTransmission(addr);
  Wire.write((reg >> 8) & 0xFF);
  Wire.write(reg & 0xFF);
  Wire.write(data);
  Wire.endTransmission();
}
